@@ -47,11 +47,19 @@ NUMBER_OF_MONITORS = user_config.get("NUMBER_OF_MONITORS", 8)
 DEFAULT_TEMP_HOST = user_config.get("DEFAULT_TEMP_HOST", "127.0.0.1")
 DEFAULT_TEMP_PORT = user_config.get("DEFAULT_TEMP_PORT", 5917)
 
+# Ramp frequency limits
+# leave it at 1, should be reasonable
+RAMP_FREQUENCY_MIN = user_config.get("RAMP_FREQUENCY_MIN", 1)
+# for high frequencies the autolock accuracy will go down
+RAMP_FREQUENCY_MAX = user_config.get("RAMP_FREQUENCY_MAX", 100)
+
+LOCK_THRESHOLD = user_config.get("LOCK_THRESHOLD", 0.8)
+LOCK_THRESHOLD_BREAK = user_config.get("LOCK_THRESHOLD_BREAK", 0.6)
+
 # ADwin variables
 SAMPLING_RATE = 200e3
-RAMP_DATA_POINTS = 0x20000
 FIFO_BUFFER_SIZE = 30003  # Buffer size that is choosen on the adwin system.
-FIFO_MAXLEN = 1000
+FIFO_MAXLEN = 3000
 NUMBER_OF_FILTERS = 5
 NUMBER_OF_SOS = 5
 
@@ -60,18 +68,17 @@ NUMBER_OF_SOS = 5
 ###########################################################################################
 PAR_TIMER = 1
 PAR_RELOADBIT = 2
-PAR_RCR = 3  # Ramp Control Register
 PAR_ACTIVE_CHANNEL = 4
 PAR_FIFOSTEPSIZE = 6
 PAR_TIMEDIFF = 7
 PAR_SENSITIVITY = 8
 # Filter Control Register, used as `10 + servo_channel`, so PAR 11-18 are also occupied
 PAR_FCR = 10
+# Lock control register 1: contains search bits, locked bits and relock bits
 PAR_LCR = 19
+# Second lock control register: contains greater bits and rampmode bits
 PAR_GCR = 20
 
-FPAR_RAMPAMP = 1  # Ramp amplitude
-
 DATA_FILTERCOEFFS = 1
 DATA_OFFSETGAIN = 2
 DATA_FIFO = 3
@@ -80,10 +87,7 @@ DATA_MONITORS = 6
 DATA_LOCK = 8
 DATA_LOCK_ITER = 9
 DATA_LOCK_OUTPUT = 10
-
-RAMP_MIN_STEPSIZE = 1
-RAMP_MAX_STEPSIZE = 255
-
+DATA_LOCK_STEPSIZE = 11
 
 ##########################################################
 ################# local settings import ##################
