@@ -18,7 +18,7 @@ def get_layout():
 app.layout = get_layout
 app.enable_dev_tools(debug=settings.DEBUG)
 if settings.DEBUG:
-    log.warning(f"Running in debug mode.")
+    log.warning("Running in debug mode.")
 ui.setCallbacks()
 server = app.server
 
