@@ -1,6 +1,9 @@
 import logging as log
+
+import numpy as np
 from dash.exceptions import PreventUpdate
 from fastnumbers import fast_real
+
 from nqontrol.general import settings
 from nqontrol.gui.dependencies import DEVICE
 
@@ -42,67 +45,8 @@ def getLockString(servo):
     lockstatus = servo.lockSearch
     locked = servo.locked
     relock = servo.relock
-    return f"search {int(lockstatus)} relock {int(relock)} locked {int(locked)}"
-
-
-def getLockRange(servo):
-    """Returns a list containing minimum and maximum value of the autolock sections RangeSlider.abs
-
-    The AutoLock options are located in the servo section.
-
-    Parameters
-    ----------
-    servo : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    obj:`list`
-        [min, max]
-
-    """
-    servo = DEVICE.servo(servo)
-    return [servo.lockSearchMin, servo.lockSearchMax]
-
-
-def getLockThreshold(servo):
-    """Returns the threshold value of the autolock.
-
-    The AutoLock options are located in the servo section.
-
-    Parameters
-    ----------
-    servo : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    obj:`float`
-        The voltage value.
-
-    """
-    servo = DEVICE.servo(servo)
-    return servo.lockThreshold
-
-
-def getLockGreater(servo):
-    """Return the initial value of the lock condition of a servo.
-    The boolean translates to greater (True) or lesser than (False) the threshold.
-    Part of the servo section.
-
-    Parameters
-    ----------
-    servo : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    :obj:`bool`
-        The lock condition as a boolean.
-
-    """
-    servo = DEVICE.servo(servo)
-    return int(servo.lockGreater)
+    rampmode = servo.lockRampmode
+    return f"search {int(lockstatus)} ramp {int(rampmode)} relock {int(relock)} locked {int(locked)}"
 
 
 def getLockRelock(servo):
@@ -115,29 +59,96 @@ def getLockRelock(servo):
 
     Returns
     -------
-    :obj:`list`
-        Empty list means `False`, element in list means `True`.
+    :obj:`bool`
+        The boolean value.
 
     """
-    result = []
-    if DEVICE.servo(servo).relock:
-        result.append("on")
-    return result
+    return DEVICE.servo(servo).relock
+
+
+def getLockAmplitude(servoNumber):
+    """Return the lock ramp amplitud for a specified :obj:`servo`. Load from save or default to 1 V.
+
+    Parameters
+    ----------
+    servoNumber : :obj:`int`
+        Servo index :obj:`servo.channel`
+
+    Returns
+    -------
+    :obj:`float`
+        Float from 0 to 10.
+    """
+    return np.round(DEVICE.servo(servoNumber).lockAmplitude, 2)
+
+
+def getLockOffset(servoNumber):
+    """Return the lock ramp offset for a specified :obj:`servo`. Load from save or default to 0 V.
+
+    Parameters
+    ----------
+    servoNumber : :obj:`int`
+        Servo index :obj:`servo.channel`
+
+    Returns
+    -------
+    :obj:`float`
+        Float value from -10 to 10.
+    """
+    return np.round(DEVICE.servo(servoNumber).lockOffset, 2)
+
+
+def getLockFrequency(servoNumber):
+    """Return the lock ramp frequency setting for specified :obj:`servo`.
+    Load from save or default to `1`.
+
+    Parameters
+    ----------
+    servoNumber : :obj:`int`
+        Servo index :obj:`servo.channel`
+
+    Returns
+    -------
+    :obj:`float`
+        Frequency as a float. Default uses frequencies from 1 to 100 Hz.
+
+    """
+    return DEVICE.servo(servoNumber).lockFrequency
+
+
+def getLockButtonLabel(servoNumber):
+    """Return the lock button label on UI start-up.
+
+    Parameters
+    ----------
+    servoNumber : :obj:`int`
+        Servo index :obj:`servo.channel`
+
+    Returns
+    -------
+    :obj:`String`
+        String description depending on lock state.
+    """
+    servo = DEVICE.servo(servoNumber)
+    if servo.lockSearch or servo.locked:
+        s = "Turn off"
+    else:
+        s = "Turn on"
+    return s
 
 
 #################### Callbacks #############################
 
 
-def callLockState(n_clicks, servoNumber):
-    """Enables the auto-lock feature on given servo.
-    Ramp is not compatible with Autolock (Essentially, the autolock should be a better ramp).
+def callLockState(ctxt, n_clicks, servoNumber):
+    """Enables the auto-lock feature on given servo on button trigger.
 
-    GUI wise, the button is located in the individual autolock section.
+    Updates the ramp and lock button labels in the UI as a return.
 
     Parameters
     ----------
     n_clicks : :obj:`bool`
-        Times the button has been clicked for toggling functionality.
+        Times the button has been clicked, only needed right now to raise PreventUpdate on startup (None trigger).
     servo : :obj:`int`
         Servo index :obj:`servo.channel`
 
@@ -149,105 +160,34 @@ def callLockState(n_clicks, servoNumber):
     """
     # check whether list is empty
     servo = DEVICE.servo(servoNumber)
-    if n_clicks is None:
+    log.debug(ctxt.triggered)
+    if len(ctxt.triggered) > 1:  # avoid ambiguous callbacks
         raise PreventUpdate()
-    current = servo.lockSearch
-    if current:
-        servo.lockSearch = 0
-    else:
-        servo.lockSearch = 1  # setting state to 1 starts searching for peak
-    return servo.lockSearch
-    # the servo autolock function will automatically disable the ramp if it is active on that channel
+    triggered = ctxt.triggered[0]["prop_id"].split(".")[0]
+    log.debug(triggered)
+    if n_clicks is not None and "Button" in triggered:
+        search = servo.lockSearch
+        locked = servo.locked
+        log.debug((search, locked))
+        if search or locked:
+            servo.lockSearch = 0
+            servo.locked = 0
+        else:
+            servo.lockSearch = 1  # setting state to 1 starts searching for peak
+
+    if servo.lockSearch or servo.locked:
+        return "Turn off"
+    return "Turn on"
 
 
-def callLockButtonLabel(servoNumber):
-    """Callback handle for the autolock button. Only sets the label of the button according to the current search status.
+def callLockThresholdInfo(analyse_clicks, servoNumber):
+    """Return a string for the UI containing information on threshold and threshold direction (`lockGreater`). Triggers the `servo.lockAnalysis` feature.
+
 
     Parameters
     ----------
-    servoNumber : :obj:`int`
-        The servo index.
-
-    Returns
-    -------
-    :obj:`String`
-        Button Label.
-    """
-    servo = DEVICE.servo(servoNumber)
-    if servo.lockSearch:
-        s = "Turn off lock"
-    else:
-        s = "Trigger lock"
-    return s
-
-
-def callLockRange(lockRange, servoNumber):
-    """Sets the search range for the autolock feature of a given servo based on UI.
-
-    Parameters
-    ----------
-    lockRange : :obj:`list`
-        Contains floats [start, end].
-    servo : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    :obj:`String`
-        Return new label containing information.
-
-    """
-    servo = DEVICE.servo(servoNumber)
-    start = lockRange[0]
-    end = lockRange[1]
-    if start > end:
-        raise PreventUpdate("Start value should be bigger.")
-    servo.lockSearchRange = [start, end]
-
-    return f"Search from {servo.lockSearchMin:.2f} V to {servo.lockSearchMax:.2f} V"
-
-
-def callLockGreater(greater, servoNumber):
-    """Return a binary value signifying the test direction for the autolock criteria.
-
-    `1`: "lock when greater than threshold,
-    `0`: "lock when below threshold"
-
-    Parameters
-    ----------
-    greater : :obj:`bool`
-        Should be a boolean, but will also translate other options to boolean, so 5 will count as True etc., according to python's bool conversion.
-    servoNumber : :obj:`int`
-        The servo index.
-
-    Returns
-    -------
-    :obj:`int`
-        The binary value of `greater`, since all callbacks need a return. Should also signify that the write to ADwin actually worked because it gets called directly from the device.
-
-    Raises
-    ------
-    PreventUpdate
-        If `greater` parameter is `None`, might happen on start-up.
-    """
-    if greater is None:
-        raise PreventUpdate()
-    servo = DEVICE.servo(servoNumber)
-    servo.lockGreater = greater
-    return servo.lockGreater
-
-
-def callLockThresholdInfo(threshold, greater, servoNumber):
-    """Return a string for the UI containing information on threshold and threshold direction (`lockGreater`). If no values are passed will get the values directly from the device.
-
-    Makes sure the current values are displayed in the UI.
-
-    Parameters
-    ----------
-    threshold : :obj:`float` or :obj:`int`
-        The autolock threshold value.
-    greater : :obj:`bool` or :obj:`int`
-        Boolean indicating the direction of the lock. Also accepts `1` or `0` instead of True/False.
+    analyse_clicks: :obj:`int`
+        Number of clicks on the analysis button. Indicates the callback trigger.
     servoNumber : :obj:`int`
         The servo index.
 
@@ -257,156 +197,88 @@ def callLockThresholdInfo(threshold, greater, servoNumber):
         The label string.
     """
     servo = DEVICE.servo(servoNumber)
-    if greater is None:
-        greater = servo.lockGreater
-    if threshold is None:
-        threshold = servo.lockThreshold
-    if greater:
+
+    # might be None at startup
+    if analyse_clicks is not None:
+        servo.lockAnalysis()
+
+    if servo.lockGreater:  # pylint: disable=protected-access
         greaterstring = ">"
     else:
         greaterstring = "<"
-    return f"Threshold {greaterstring}{threshold:.2f} V"
+    return f"Threshold {greaterstring}{servo.lockThreshold:.2f} V (Break {servo.lockThresholdBreak:.2f} V)"
 
 
-def callLockRelock(values, servo):
-    """Set whether the AutoLock should relock automatically whenever falling
-    above/below threshold for a given servo.
+def callLockRelock(value: bool, servoNumber: int):
+    """Set whether the AutoLock should relock automatically whenever falling above/below threshold for a given servo.
 
     Parameters
     ----------
-    values : :obj:`list`
-        As with all Dash checklists, even though this is for a single element,
-        the callback input is a list. Empty list means off, none-empty means on.
-    servoNumber : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    :obj:`bool`
-        The relock value, since the UI requires a return.
-
-    """
-    servo = DEVICE.servo(servo)
-    if values:
-        servo.relock = True
-    else:
-        servo.relock = False
-    return servo.relock
-
-
-def callLockThreshold(threshold, servo):
-    """Set the autolock threshold value for a servo.
-
-    Parameters
-    ----------
-    value : :obj:`float`
-        The threshold value.
-    servoNumber : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    :obj:`int`
-        The relock value, since the UI requires a return.
-
-    """
-    try:
-        threshold = fast_real(threshold, raise_on_invalid=True)
-    except (ValueError, TypeError):
-        raise PreventUpdate("Input must be a real number.")
-    if not -10 <= threshold <= 10:
-        log.warning(f"Must be a value between -10 and 10, was {threshold}")
-        raise PreventUpdate(f"Must be a value between -10 and 10, was {threshold}")
-    servo = DEVICE.servo(servo)
-    servo.lockThreshold = threshold
-    return servo.lockThreshold
-
-
-############################################################
-######################## Ramp Widget #######################
-############################################################
-
-
-#################### Getters ###############################
-
-
-def getServoAmplitude(servoNumber):
-    """Return the ramp amplitude setting for the specified :obj:`servo`.
-    Load from save or default to `0.1`.
-
-    Concerns the servo ramp section of the UI.
-
-    Parameters
-    ----------
-    servoNumber : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    :obj:`float`
-        Amplitude as float.
-
-    """
-    servo = DEVICE.servo(servoNumber)
-    amplitude = servo.rampAmplitude
-    return amplitude
-
-
-def getServoFrequency(servoNumber):
-    """Return the ramp frequency setting for specified :obj:`servo`.
-    Load from save or defaukt to `20`.
-
-    Concerns the servo ramp section of the UI.
-
-    Parameters
-    ----------
-    servoNumber : :obj:`int`
-        Servo index :obj:`servo.channel`
-
-    Returns
-    -------
-    type
-        Description of returned object.
-
-    """
-    servo = DEVICE.servo(servoNumber)
-    frequency = servo.rampFrequency
-    return frequency
-
-
-#################### Callbacks #############################
-
-
-def callRamp(amp, freq, context, servoNumber):
-    """Send ramp parameters entered in servo control section of the UI to
-    the corresponding :obj:`nqontrol.Servo`.
-
-    Parameters
-    ----------
-    amp : :obj:`float`
-        Ramp amplitude.
-    freq : :obj:`float`
-        Ramp frequency.
-    context : :obj:'json'
-        Dash callback context. Please check the dash docs for more info.
+    value : :obj:`bool`
+        The toggle value.
     servoNumber : :obj:`int`
         Servo index :obj:`servo.channel`
 
     Returns
     -------
     :obj:`String`
-        UI label string describing current ramp state.
+        The value in a string description, since the UI requires a return.
+
+    """
+    servo = DEVICE.servo(servoNumber)
+    # we could theoretically just check if the list contained anything, but it's more descriptive like this
+    servo.relock = value
+    return f"Relock: {int(servo.relock)}"
+
+
+def callLockRamp(amplitude, offset, freq, context, servoNumber):
+    """Send lock ramping parameters entered in servo control section of the UI to
+    the corresponding :obj:`nqontrol.Servo`.
+
+    Parameters
+    ----------
+    amplitude : :obj:`float`
+        Lock ramp search amplitude.
+    offset : :obj:`float`
+        Lock ramp search offset.
+    freq : :obj:`float`
+        Lock ramp frequency.
+    context : :obj:'json'
+        Dash callback context. Please check the dash docs for more info. Used to determine input.
+    servoNumber : :obj:`int`
+        Servo index :obj:`servo.channel`
+
+    Returns
+    -------
+    :obj:`String`
+        UI label string describing current lock ramp state.
 
     """
     servo = DEVICE.servo(servoNumber)
     triggered = context.triggered[0]["prop_id"].split(".")[0]
-    if "ramp_amp" in triggered:
-        servo.rampAmplitude = amp
-    if "ramp_freq" in triggered:
-        servo.rampFrequency = freq
-    amp = servo.rampAmplitude
-    freq = servo.rampFrequency
-    return f"Amplitude: {amp:.2f} V | Frequency: {freq:.2f} Hz"
+    log.debug(context.triggered)
+    if "lock_amplitude" in triggered:
+        servo.lockAmplitude = amplitude
+    if "lock_offset" in triggered:
+        servo.lockOffset = offset
+    if "lock_freq" in triggered:
+        servo.lockFrequency = freq
+    return f"Amplitude {servo.lockAmplitude:.2f} V | Offset {servo.lockOffset:.2f} V | Frequency {servo.lockFrequency:.2f} Hz"
+
+
+def callLockMode(mode: bool, servoNumber: int):
+    """Toggles mode of the autolock section. Since the ToggleSwitch only has a boolean value, `False` ist mapped to "rampmode" and `True` is mapped to lockmode.
+
+    Parameters
+    ----------
+    mode : bool
+        The incoming toggle switch value.
+    servoNumber : int
+        Number of the servo channel.
+    """
+    servo = DEVICE.servo(servoNumber)
+    servo.lockRampmode = not mode
+    return not mode
 
 
 ############################################################
